//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "common.h"

QString randPickItem(const QStringList &list)
{
    if (!list.isEmpty())
    {
        if (list.size() == 1)
        {
            return list[0];
        }
        else
        {
            return list[QRandomGenerator().bounded(0, list.size() - 1)];
        }
    }

    return QString();
}

void elasticWid(QWidget *wid)
{
    wid->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    wid->setContentsMargins(0,0,0,0);
}

void elasticWidLimitH(QWidget *wid)
{
    wid->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    wid->setContentsMargins(0,0,0,0);
}

QStringList lsFiles(const QString &path, const QStringList &filters)
{
    QDir dirObj(path);

    dirObj.setFilter(QDir::Files);
    dirObj.setSorting(QDir::Name);

    if (!filters.isEmpty())
    {
        dirObj.setNameFilters(filters);
    }

    return dirObj.entryList();
}

QStringList lsFiles(const QString &path)
{
    return lsFiles(path, QStringList());
}

QStringList lsVidFiles(const QString &path)
{
    QStringList filters;

    filters << "*.mp4"; filters << "*.ts";  filters << "*.mov";  filters << "*.avi";
    filters << "*.wmv"; filters << "*.flv"; filters << "*.webm"; filters << "*.avchd";
    filters << "*.mkv";

    return lsFiles(path, filters);
}

QStringList lsImgFiles(const QString &path)
{
    QStringList filters;

    filters << "*.bmp"; filters << "*.jpeg";  filters << "*.jpg";  filters << "*.gif";
    filters << "*.png";

    return lsFiles(path, filters);
}

QStringList lsConfFiles(const QString &path)
{
    QStringList filters;

    filters << "*.conf";

    return lsFiles(path, filters);
}

QStringList lsDirsInDir(const QString &path)
{
    QDir dirObj(path);

    dirObj.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    dirObj.setSorting(QDir::Name);

    return dirObj.entryList();
}

void rdLine(const QString &param, const QString &line, QString *value)
{
    if (line.startsWith(param))
    {
        *value = line.mid(param.size()).trimmed();
    }
}

void rdLine(const QString &param, const QString &line, int *value)
{
    if (line.startsWith(param))
    {
        *value = line.mid(param.size()).trimmed().toInt();
    }
}

void rdLine(const QString &param, const QString &line, bool *value)
{
    if (line.startsWith(param))
    {
        auto val = line.mid(param.size()).trimmed();
        
        *value = (val == "y" || val == "Y");
    }
}

bool delFolderContents(const QString &path)
{
    auto ret   = false;
    auto files = lsFiles(path);
    auto dirs  = lsDirsInDir(path);

    for (auto &&file: files)
    {
        ret = QFile::remove(path + QDir::separator() + file);
    }

    for (auto &&dir: dirs)
    {
        ret = QDir(dir).removeRecursively();
    }

    return ret;
}

void extCorrection(QString &ext)
{
    if (!ext.startsWith("."))
    {
        ext = "." + ext;
    }
}

bool mkPath(const QString &path)
{
    auto ret = true;
    
    if (!QFileInfo::exists(path))
    {
        ret = QDir(path).mkpath(path);
    }
    
    return ret;
}

void resetSharedRes(shared_t *share)
{
    share->camName.clear();
    share->buffPath.clear();
    share->recPath.clear();
    share->playPath.clear();

    share->conf = share->appConfPath;
}

bool runCmd(const QStringList &args, QWidget *parent)
{
    auto ret = true;

    if (!args.isEmpty())
    {
        if (QProcess::execute(args[0], args.mid(1)) != 0)
        {
            QMessageBox::critical(parent, QObject::tr("Exec Error"), QObject::tr("Command: ") + args.join(' ') + QObject::tr(" returned non-zero. See Help > Logs for details"));

            ret = false;
        }
    }

    return ret;
}

bool rdConf(const QString &filePath, shared_t *share)
{
    QFile varFile(filePath);

    if (!varFile.open(QFile::ReadOnly))
    {
        share->retCode = ENOENT;

        QTextStream(stderr) << "err: config file - " << filePath << " does not exists or lack read permissions." << Qt::endl;
    }
    else
    {
        resetSharedRes(share);

        share->conf = filePath;

        QString line;

        do
        {
            line = QString::fromUtf8(varFile.readLine());

            if (!line.startsWith("#"))
            {
                rdLine("host_addr = ",   line, &share->hostAddr);
                rdLine("host_port = ",   line, &share->hostPort);
                rdLine("host_key = ",    line, &share->hostKey);
                rdLine("host_user = ",   line, &share->hostUser);
                rdLine("cam_name = ",    line, &share->camName);
                rdLine("buffer_path = ", line, &share->buffPath);
                rdLine("rec_path = ",    line, &share->recPath);
                rdLine("play_path = ",   line, &share->playPath);
                rdLine("winrect_w = ",   line, &share->windowW);
                rdLine("winrect_h = ",   line, &share->windowH);
                rdLine("winrect_x = ",   line, &share->windowX);
                rdLine("winrect_y = ",   line, &share->windowY);
                rdLine("player_cmd = ",  line, &share->playerCmd);
            }

        } while(!line.isEmpty());

        if (share->camName.isEmpty())
        {
            share->camName = QFileInfo(share->conf).baseName();
        }

        if (share->buffPath.isEmpty())
        {
            share->buffPath = "/tmp/jmotion/" + share->camName;
        }
        else
        {
            share->buffPath = QDir::cleanPath(share->buffPath);
        }

        if (share->recPath.isEmpty())
        {
            share->recPath = "/var/jmotion/footage/" + share->camName;
        }
        else
        {
            share->recPath = QDir::cleanPath(share->recPath);
        }
    }

    return share->retCode == 0;
}

void wrConf(const QString &filePath, shared_t *share)
{
    QFile       file(filePath);
    QString     txt;
    QTextStream ts(&txt);

    if (file.open(QFile::WriteOnly))
    {
        if (filePath == share->appConfPath)
        {
            ts << "winrect_w = "  << share->windowW   << Qt::endl;
            ts << "winrect_h = "  << share->windowH   << Qt::endl;
            ts << "winrect_x = "  << share->windowX   << Qt::endl;
            ts << "winrect_y = "  << share->windowY   << Qt::endl;
            ts << "player_cmd = " << share->playerCmd << Qt::endl;
        }
        else
        {
            ts << "host_addr = " << share->hostAddr << Qt::endl;
            ts << "host_port = " << share->hostPort << Qt::endl;
            ts << "host_key = "  << share->hostKey  << Qt::endl;
            ts << "host_user = " << share->hostUser << Qt::endl;
        }

        file.write(txt.toUtf8());
    }

    file.close();
}

void createOrUpdateRecent(shared_t *share)
{
    if (share->conf == share->appConfPath)
    {
        QCryptographicHash hasher(QCryptographicHash::Md5);

        hasher.addData(share->playPath.toUtf8());

        wrConf(share->recentsPath + QDir::separator() + hasher.result().toHex() + ".conf", share);
    }
    else
    {
        wrConf(share->conf, share);
    }
}

void buildGrid(QGridLayout *mainLayout, int parentWidth, const QList<QWidget*> &widList, QList<QWidget*> &spcList, int cellWidth, QWidget *parentWid)
{
    auto columns = 0;

    if (widList.size() == 1)
    {
        columns = 1;
    }
    else
    {
        columns = parentWidth / cellWidth;
    }

    for (auto &&wid : widList)
    {
        mainLayout->removeWidget(wid);
    }

    for (auto &&wid : spcList)
    {
        mainLayout->removeWidget(wid);

        wid->deleteLater();
    }

    spcList.clear();

    auto rows = 0;

    if (columns != 0)
    {
        for (auto row = 0, i = 0; i < widList.size(); ++row, ++rows)
        {
            for (auto col = 0; (col < columns) && (i < widList.size()); ++col, ++i)
            {
                mainLayout->addWidget(widList[i], row, col);
            }

            auto spacer = new QWidget(mainLayout->widget());

            elasticWid(spacer);

            mainLayout->addWidget(spacer, row, columns);

            spcList.append(spacer);
        }
    }
    else
    {
        qInfo() << "buildGrid() << calculated a zero column grid. This may cause a blank layout.";
    }

    parentWid->setFixedHeight((rows * THUMB_H) + 10);
}

QStringList parseArgs(const QByteArray &data, int maxArgs, int *pos)
{
    QStringList ret;
    QString     arg;

    auto line      = QString::fromUtf8(data);
    auto inDQuotes = false;
    auto inSQuotes = false;
    auto escaped   = false;

    if (pos != nullptr) *pos = 0;

    for (int i = 0; i < line.size(); ++i)
    {
        if (pos != nullptr) *pos += 1;

        if ((line[i] == '\'') && !inDQuotes && !escaped)
        {
            // single quote '

            inSQuotes = !inSQuotes;
        }
        else if ((line[i] == '\"') && !inSQuotes && !escaped)
        {
            // double quote "

            inDQuotes = !inDQuotes;
        }
        else
        {
            escaped = false;

            if (line[i].isSpace() && !inDQuotes && !inSQuotes)
            {
                // space

                if (!arg.isEmpty())
                {
                    ret.append(arg);
                    arg.clear();
                }
            }
            else
            {
                if ((line[i] == '\\') && ((i + 1) < line.size()))
                {
                    if ((line[i + 1] == '\'') || (line[i + 1] == '\"'))
                    {
                        escaped = true;
                    }
                    else
                    {
                        arg.append(line[i]);
                    }
                }
                else
                {
                    arg.append(line[i]);
                }
            }
        }

        if ((ret.size() >= maxArgs) && (maxArgs != -1))
        {
            break;
        }
    }

    if (!arg.isEmpty() && !inDQuotes && !inSQuotes)
    {
        ret.append(arg);
    }

    return ret;
}

Proc::Proc(const QString &cmdLine, QObject *parent) : QProcess(parent)
{
    args = parseArgs(cmdLine.toUtf8());

    connect(this, &Proc::readyReadStandardError,  this, &Proc::rdStdErr);
    connect(this, &Proc::readyReadStandardOutput, this, &Proc::rdStdOut);
}

void Proc::rdStdErr()
{
    stdErr += readAllStandardError().trimmed();
}

void Proc::rdStdOut()
{
    stdOut += readAllStandardOutput().trimmed();
}

int Proc::exec()
{
    if (args.isEmpty())
    {
        stdErr = tr("Command line not parsable"); return 1;
    }
    else
    {
        setProgram(args[0]);
        setArguments(args.mid(1));

        start();
        waitForFinished();

        return exitCode();
    }
}
