//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "main_widget.h"
#include "host_dialog.h"

MainGui::MainGui(shared_t *share, QWidget *parent) : QMainWindow(parent)
{
    shared = share;

    layout()->setSpacing(0);
}

void MainGui::closeEvent(QCloseEvent *event)
{
    if (!shared->playPath.isEmpty())
    {
        thread()->sleep(1);

        QString cmdA = "umount " + shared->playPath;
        QString cmdB = "rmdir "  + shared->playPath;

        system(cmdA.toUtf8().data());
        system(cmdB.toUtf8().data());
    }

    QWidget::closeEvent(event);
}

MainWidget::MainWidget(shared_t *share, MainGui *parent) : QWidget(parent)
{
    shared     = share;
    mainGui    = parent;
    sshOpts    = new ssh_opt();
    homeLayout = new QVBoxLayout(this);

    genConnectWidget();
    genSettingsWidget();
    genSpacer();

    parentWidget()->setWindowTitle(QString(APP_NAME));
}

void MainWidget::closeEvent(QCloseEvent *event)
{
    close();

    QWidget::closeEvent(event);
}

void MainWidget::genConnectWidget()
{
    auto wid = new QGroupBox(this);
    auto lay = new QHBoxLayout(wid);

    hostAddr   = new QComboBox(this);
    sshOptBtn  = new QPushButton(tr("Options"), this);
    connectBtn = new QPushButton(tr("Connect"), this);
    closeBtn   = new QPushButton(tr("Close"), this);
    recents    = new Recents(shared, hostAddr, sshOpts, this);

    hostAddr->setEditable(true);
    hostAddr->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    closeBtn->setVisible(false);

    wid->setTitle(tr("Host"));
    lay->addWidget(new QLabel(tr("Host Address: "), this));
    lay->addWidget(hostAddr);
    lay->addStretch();
    lay->addWidget(sshOptBtn);
    lay->addWidget(connectBtn);
    lay->addWidget(closeBtn);

    homeLayout->addWidget(wid);

    connect(connectBtn, &QPushButton::clicked, this, &MainWidget::open);
    connect(closeBtn,   &QPushButton::clicked, this, &MainWidget::close);
    connect(sshOptBtn,  &QPushButton::clicked, this, &MainWidget::openSshOptsDialog);
}

void MainWidget::genSettingsWidget()
{
    auto wid    = new QGroupBox(this);
    auto lay    = new QHBoxLayout(wid);
    auto defBtn = new QPushButton(tr("Default"), this);

    playerCmd = new QLineEdit(this);

    playerCmd->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    playerCmd->setText(shared->playerCmd);

    wid->setTitle(tr("Settings"));
    lay->addWidget(new QLabel(tr("Video Player Command: "), this));
    lay->addWidget(playerCmd);
    lay->addStretch();
    lay->addWidget(defBtn);

    homeLayout->addWidget(wid);

    connect(defBtn,    &QPushButton::clicked,  this, &MainWidget::resetPlayerDefault);
    connect(playerCmd, &QLineEdit::textEdited, this, &MainWidget::updatePlayerCmd);
}

void MainWidget::genSpacer()
{
    spacer = new QWidget(this);

    elasticWid(spacer);

    homeLayout->addWidget(spacer);
}

void MainWidget::resetPlayerDefault()
{
    playerCmd->setText(DEFAULT_PLAYER);

    shared->playerCmd = DEFAULT_PLAYER;
}

void MainWidget::updatePlayerCmd(const QString &line)
{
    shared->playerCmd = line;
}

void MainWidget::close()
{
    parentWidget()->setWindowTitle(QString(APP_NAME));

    closeBtn->setVisible(false);
    connectBtn->setVisible(true);
    sshOptBtn->setVisible(true);

    sshOpts->key.clear();
    sshOpts->pw.clear();
    sshOpts->port.clear();
    sshOpts->user.clear();

    shared->hostAddr.clear();
    shared->hostKey.clear();
    shared->hostPort.clear();
    shared->hostUser.clear();

    liveWid->close();
    footageWid->close();

    if (!shared->playPath.isEmpty())
    {
        thread()->sleep(1);

        QString cmdA = "umount " + shared->playPath;
        QString cmdB = "rmdir "  + shared->playPath;

        system(cmdA.toUtf8().data());
        system(cmdB.toUtf8().data());
    }

    shared->playPath.clear();
}

void MainWidget::openSshOptsDialog()
{
    HostDialog(sshOpts, this).exec();
}

void MainWidget::open()
{
    shared->hostAddr = hostAddr->currentText();
    shared->hostKey  = sshOpts->key;
    shared->hostPort = sshOpts->port;
    shared->hostUser = sshOpts->user;
    shared->playPath = QDir::tempPath() + QDir::separator() + shared->hostAddr;

    createOrUpdateRecent(shared);

    mkPath(shared->playPath);

    QString sshfsCmd = "sshfs";

    if (shared->hostUser.isEmpty()) sshfsCmd += " ";
    else                            sshfsCmd += " " + shared->hostUser + "@";

    sshfsCmd += shared->hostAddr + ":/ " + shared->playPath;

    if (!shared->hostPort.isEmpty()) sshfsCmd += " -p "              + shared->hostPort;
    if (!shared->hostKey.isEmpty())  sshfsCmd += " -o IdentityFile=" + shared->hostKey;

    if (!sshOpts->pw.isEmpty())
    {
        sshfsCmd = "sshpass -p " + sshOpts->pw + " " + sshfsCmd;
    }

    Proc proc(sshfsCmd, this);

    if (proc.exec() == 0)
    {
        parentWidget()->setWindowTitle(shared->hostAddr + " - " + QString(APP_NAME));

        closeBtn->setVisible(true);
        connectBtn->setVisible(false);
        sshOptBtn->setVisible(false);
        spacer->close();

        liveWid    = new QGroupBox(this);
        footageWid = new QGroupBox(this);

        liveWid->setTitle(tr("Live"));
        footageWid->setTitle(tr("Footage"));

        auto liveLayout    = new QVBoxLayout(liveWid);
        auto footageLayout = new QVBoxLayout(footageWid);

        liveLayout->addWidget(new PlaylistWidget(shared, CAM_LIVE, liveWid));
        footageLayout->addWidget(new PlaylistWidget(shared, CAM_FOOTAGE, footageWid));

        homeLayout->addWidget(liveWid);
        homeLayout->addWidget(footageWid);

        genSpacer();
    }
    else
    {
        auto msg = proc.errorString().trimmed();

        if (msg.isEmpty() || msg == tr("Unknown error"))
        {
            msg = proc.stdErr;
        }

        if (msg.isEmpty())
        {
            msg = proc.stdOut;
        }

        if (msg.isEmpty())
        {
            msg = tr("Failed to connect to host via SSH - unkown error.");
        }

        msg = msg + tr(" - return code: ") + QString::number(proc.exitCode());

        QMessageBox::critical(this, APP_NAME, msg);
    }
}
