//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "playlist_widget.h"

PlaylistItem::PlaylistItem(shared_t *appConf, const QString &path, const QString &confPath, QWidget *parent) : QLabel(parent)
{
    setObjectName(QFileInfo(path).baseName());

    popMsg      = new QDialog(this, Qt::Window | Qt::FramelessWindowHint | Qt::Tool);
    popMsgText  = new QLabel(popMsg);
    confChecker = new QTimer(this);
    proc        = new QProcess(this);
    basePath    = QDir::cleanPath(path);
    conf        = confPath;
    appShare    = appConf;

    popMsg->hide();
    popMsg->setModal(false);
    confChecker->setSingleShot(false);
    confChecker->setInterval(5000);
    confChecker->start();
    popMsg->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    auto popLayout = new QVBoxLayout(popMsg);

    popLayout->addWidget(popMsgText, 0, Qt::AlignCenter);

    setCursor(Qt::PointingHandCursor);
    setFixedWidth(THUMB_W);
    setFixedHeight(THUMB_H);

    genImgAndText();
    setMouseTracking(true);

    connect(confChecker, &QTimer::timeout, this, &PlaylistItem::checkConfExists);
}

void PlaylistItem::checkConfExists()
{
    if (!QFileInfo::exists(conf))
    {
        emit rmItem(conf, this);
    }
}

void PlaylistItem::loadImgFromFile(QString path)
{
    if (path.isEmpty() || !QFileInfo::exists(path))
    {
        path = ":/img/no_mdeia.png";
    }

    auto pix = QPixmap(path);

    setPixmap(pix.scaled(THUMB_W, THUMB_H, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
}

void PlaylistItem::genImgAndText()
{
    auto imgDir  = basePath + QDir::separator() + "img" + QDir::separator();
    auto imgList = lsImgFiles(imgDir);
    auto imgFile = randPickItem(imgList);

    loadImgFromFile(imgDir + imgFile);

    popMsgText->setText(objectName());
}

void PlaylistItem::leaveEvent(QEvent *event)
{
    Q_UNUSED(event);

    popMsg->hide();
}

void PlaylistItem::enterEvent(QEnterEvent *event)
{
    Q_UNUSED(event);

    dispText();
}

void PlaylistItem::playerExited()
{

}

void PlaylistItem::mouseReleaseEvent(QMouseEvent *event)
{
    Q_UNUSED(event);

    auto args = parseArgs(appShare->playerCmd.toUtf8());

    if (args.isEmpty())
    {
        QMessageBox::critical(this, APP_NAME, tr("player command not parsable"));
    }
    else if (!args.contains("pls.m3u8"))
    {
        QMessageBox::critical(this, APP_NAME, tr("missing argument in player command for pls.m3u8"));
    }
    else
    {
        if (proc->state() == QProcess::Running)
        {
            proc->kill();
            proc->waitForFinished();
        }

        connect(proc, &QProcess::finished, this, &PlaylistItem::playerExited);

        proc->setWorkingDirectory(basePath);
        proc->setProgram(args[0]);
        proc->setArguments(args.mid(1));
        proc->start();
    }
}

void PlaylistItem::dispText()
{
    auto pnt = mapToGlobal(QPointF(0, THUMB_H / 2));

    popMsg->move(pnt.x(), pnt.y());
    popMsg->show();
}

PlaylistWidget::PlaylistWidget(shared_t *share, play_item_t type, QWidget *parent) : QScrollArea(parent)
{
    auto listWidget = new QWidget(this);

    shared     = share;
    itemType   = type;
    listLayout = new QGridLayout(listWidget);

    elasticWid(listWidget);

    setWidget(listWidget);
    setWidgetResizable(true);
    setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setFrameStyle(QFrame::NoFrame);

    scan();
}

void PlaylistWidget::scan()
{
    auto mntPath  = QDir::cleanPath(shared->playPath) + QDir::separator();
    auto confPath = mntPath + "etc" + QDir::separator() + "jmotion" + QDir::separator();
    auto confList = lsFiles(confPath);

    for (auto conf : confList)
    {
        if (!fileList.contains(conf))
        {
            shared_t      confParams;
            PlaylistItem *item;

            rdConf(confPath + conf, &confParams);

            if (itemType == CAM_LIVE)
            {
                item = new PlaylistItem(shared, mntPath + confParams.buffPath, confPath + conf, this);
            }
            else
            {
                item = new PlaylistItem(shared, mntPath + confParams.recPath, confPath + conf, this);
            }

            connect(item, &PlaylistItem::rmItem, this, &PlaylistWidget::rmItem);

            fileList.append(conf);
            widList.append(item);
        }
    }

    reGrid();
}

void PlaylistWidget::reGrid()
{
    buildGrid(listLayout, width(), widList, spcList, THUMB_W, this);

    parentWidget()->setFixedHeight(size().height() * 2);
}

void PlaylistWidget::rmItem(const QString &confPath, const QWidget *item)
{
    fileList.removeOne(confPath);
    widList.removeOne(item);

    reGrid();
}

void PlaylistWidget::showEvent(QShowEvent *event)
{
    reGrid(); QScrollArea::showEvent(event);
}

void PlaylistWidget::resizeEvent(QResizeEvent *event)
{
    reGrid(); QScrollArea::resizeEvent(event);
}
